/*
 * Copyright 2013 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package ratpack.manual

import com.google.common.base.StandardSystemProperty
import ratpack.manual.snippets.CodeSnippetTestCase
import ratpack.manual.snippets.CodeSnippetTests
import ratpack.manual.snippets.executer.GroovySnippetExecuter
import ratpack.manual.snippets.executer.JavaSnippetExecuter
import ratpack.manual.snippets.executer.SnippetExecuter
import ratpack.manual.snippets.extractor.JavadocSnippetExtractor
import ratpack.manual.snippets.fixture.*

class JavadocCodeSnippetTests extends CodeSnippetTestCase {

  public static final LinkedHashMap<String, SnippetExecuter> FIXTURES = [
    "tested"            : new GroovySnippetExecuter(true, new GroovyScriptFixture()),
    "tested-dynamic"    : new GroovySnippetExecuter(false, new GroovyScriptFixture()),
    "java-chain-dsl"    : new JavaSnippetExecuter(new JavaChainDslFixture()),
    "groovy-chain-dsl"  : new GroovySnippetExecuter(true, new GroovyChainDslFixture()),
    "groovy-ratpack-dsl": new GroovySnippetExecuter(true, new GroovyRatpackDslNoRunFixture()),
    "java"              : new JavaSnippetExecuter(new SnippetFixture()),
    "java-args"         : new JavaSnippetExecuter(new SnippetFixture(), "thing.name=foo"),
  ]

  @Override
  protected void addTests(CodeSnippetTests tests) {
    File cwd = new File(StandardSystemProperty.USER_DIR.value())
    File root
    if (new File(cwd, "ratpack-manual.gradle").exists()) {
      root = cwd.parentFile
    } else {
      root = cwd
    }

    root.eachDirMatch(~/ratpack-.+/) {
      def mainSrc = new File(it, "src/main")
      if (mainSrc.exists()) {
        FIXTURES.each { selector, executer ->
          JavadocSnippetExtractor.extract(mainSrc, "**/*.java", selector, executer).each {
            tests.add(it)
          }
        }
      }
    }
  }
}
